@@ -41,7 +41,8 @@ class StatsigServer:
             self.__statsig_metadata = _StatsigMetadata.get()
             self._network = _StatsigNetwork(sdkKey, options)
             self._logger = _StatsigLogger(
-                self._network, self.__shutdown_event, self.__statsig_metadata, self._errorBoundary, options.local_mode, options.event_queue_size)
+                self._network, self.__shutdown_event, self.__statsig_metadata, self._errorBoundary, options.local_mode,
+                options.event_queue_size)
             self._evaluator = _Evaluator()
 
             self._last_update_time = 0
@@ -52,14 +53,16 @@ class StatsigServer:
                 else:
                     self._download_config_specs()
                 self.__background_download_configs = threading.Thread(
-                    target=self._sync, args=(self._download_config_specs, options.rulesets_sync_interval or RULESETS_SYNC_INTERVAL,))
+                    target=self._sync,
+                    args=(self._download_config_specs, options.rulesets_sync_interval or RULESETS_SYNC_INTERVAL,))
                 self.__background_download_configs.daemon = True
                 self.__background_download_configs.start()
 
             if not options.local_mode:
                 self._download_id_lists()
                 self.__background_download_idlists = threading.Thread(
-                    target=self._sync, args=(self._download_id_lists, options.idlists_sync_interval or IDLISTS_SYNC_INTERVAL,))
+                    target=self._sync,
+                    args=(self._download_id_lists, options.idlists_sync_interval or IDLISTS_SYNC_INTERVAL,))
                 self.__background_download_idlists.daemon = True
                 self.__background_download_idlists.start()
 
@@ -93,6 +96,7 @@ class StatsigServer:
     def get_experiment(self, user: StatsigUser, experiment_name: str):
         def task():
             return self.get_config(user, experiment_name)
+
         return self._errorBoundary.capture(task, lambda: DynamicConfig({}, experiment_name, ""))
 
     def get_layer(self, user: StatsigUser, layer_name: str) -> Layer:
@@ -127,7 +131,6 @@ class StatsigServer:
             event.user = self.__normalize_user(event.user)
             self._logger.log(event)
 
-
         self._errorBoundary.swallow(task)
 
     def shutdown(self):
@@ -152,6 +155,15 @@ class StatsigServer:
         self._errorBoundary.swallow(
             lambda: self._evaluator.override_config(experiment, value, user_id))
 
+    def get_client_initialize_response(self, user: StatsigUser):
+        def task():
+            return self._evaluator.get_client_initialize_response(self.__normalize_user(user))
+
+        def recover():
+            return None
+
+        return self._errorBoundary.capture(task, recover)
+
     def evaluate_all(self, user: StatsigUser):
         def task():
             all_gates = dict()
@@ -227,7 +239,8 @@ class StatsigServer:
             if network_config is None:
                 return _ConfigEvaluation()
 
-            return _ConfigEvaluation(json_value=network_config.get("value", {}), rule_id=network_config.get("ruleID", ""))
+            return _ConfigEvaluation(json_value=network_config.get("value", {}),
+                                     rule_id=network_config.get("ruleID", ""))
         elif log_exposure:
             if not is_layer:
                 self._logger.log_config_exposure(
@@ -248,8 +261,8 @@ class StatsigServer:
                 sync_func()
             except Exception as e:
                 self._errorBoundary.log_exception(e)
-           
-    def _bootstrap_config_specs(self,):
+
+    def _bootstrap_config_specs(self, ):
         if self._options.bootstrap_values is None:
             return
         try:
@@ -279,7 +292,7 @@ class StatsigServer:
         })
         self.__save_json_config_specs(specs, True)
 
-    def _download_id_list(self, url, list_name, all_lists, start_index):
+    def _download_id_list(self, url, list_name, local_list, all_lists, start_index):
         resp = self._network.get_request(
             url, headers={"Range": "bytes=%s-" % start_index})
         if resp is None:
@@ -292,7 +305,6 @@ class StatsigServer:
             content = resp.text
             if content is None:
                 return
-            list = all_lists[list_name]
             first_char = content[0]
             if first_char != "+" and first_char != "-":
                 raise StatsigNameError("Seek range invalid.")
@@ -303,66 +315,69 @@ class StatsigServer:
                 op = line[0]
                 id = line[1:].strip()
                 if op == "+":
-                    list.get("ids", set()).add(id)
+                    local_list.get("ids", set()).add(id)
                 elif op == "-":
-                    list.get("ids", set()).discard(id)
-            list["readBytes"] = start_index + content_length
+                    local_list.get("ids", set()).discard(id)
+            local_list["readBytes"] = start_index + content_length
+            all_lists[list_name] = local_list
         except Exception as e:
-            # something went wrong with the content, reset the list to start over time next
-            all_lists.pop(list_name, None)
+            self._errorBoundary.log_exception(e)
 
     def _download_id_lists(self):
-        server_id_lists = self._network.post_request("get_id_lists", {
-            "statsigMetadata": self.__statsig_metadata,
-        })
-        if server_id_lists is None:
-            return
+        try: 
+            server_id_lists = self._network.post_request("get_id_lists", {
+                "statsigMetadata": self.__statsig_metadata,
+            })
+            if server_id_lists is None:
+                return
 
-        local_id_lists = self._evaluator.get_id_lists()
-        thread_pool = []
+            local_id_lists = self._evaluator.get_id_lists()
+            thread_pool = []
 
-        for list_name in server_id_lists:
-            server_list = server_id_lists.get(list_name, dict())
-            url = server_list.get("url", None)
-            size = server_list.get("size", 0)
-            local_list = local_id_lists.get(list_name, dict())
+            for list_name in server_id_lists:
+                server_list = server_id_lists.get(list_name, dict())
+                url = server_list.get("url", None)
+                size = server_list.get("size", 0)
+                local_list = local_id_lists.get(list_name, dict())
 
-            new_creation_time = server_list.get("creationTime", 0)
-            old_creation_time = local_list.get("creationTime", 0)
-            new_file_id = server_list.get("fileID", None)
-            old_file_id = local_list.get("fileID", "")
+                new_creation_time = server_list.get("creationTime", 0)
+                old_creation_time = local_list.get("creationTime", 0)
+                new_file_id = server_list.get("fileID", None)
+                old_file_id = local_list.get("fileID", "")
 
-            if url is None or new_creation_time < old_creation_time or new_file_id is None:
-                continue
+                if url is None or new_creation_time < old_creation_time or new_file_id is None:
+                    continue
 
-            # should reset the list if a new file has been created
-            if new_file_id != old_file_id and new_creation_time >= old_creation_time:
-                local_list = {
-                    "ids": set(),
-                    "readBytes": 0,
-                    "url": url,
-                    "fileID": new_file_id,
-                    "creationTime": new_creation_time,
-                }
-                local_id_lists[list_name] = local_list
-            read_bytes = local_list.get("readBytes", 0)
-            # check if read bytes count is the same as total file size; only download additional ids if sizes don't match
-            if size <= read_bytes or url == "":
-                continue
-            thread = threading.Thread(
-                target=self._download_id_list, args=(url, list_name, local_id_lists, read_bytes, ))
-            thread.daemon = True
-            thread_pool.append(thread)
-            thread.start()
+                # should reset the list if a new file has been created
+                if new_file_id != old_file_id and new_creation_time >= old_creation_time:
+                    local_list = {
+                        "ids": set(),
+                        "readBytes": 0,
+                        "url": url,
+                        "fileID": new_file_id,
+                        "creationTime": new_creation_time,
+                    }
+                    
+                read_bytes = local_list.get("readBytes", 0)
+                # check if read bytes count is the same as total file size; only download additional ids if sizes don't match
+                if size <= read_bytes or url == "":
+                    continue
+                thread = threading.Thread(
+                    target=self._download_id_list, args=(url, list_name, local_list, local_id_lists, read_bytes, ))
+                thread.daemon = True
+                thread_pool.append(thread)
+                thread.start()
 
-        for thread in thread_pool:
-            thread.join()
+            for thread in thread_pool:
+                thread.join()
 
-        deleted_lists = []
-        for list_name in local_id_lists:
-            if list_name not in server_id_lists:
-                deleted_lists.append(list_name)
+            deleted_lists = []
+            for list_name in local_id_lists:
+                if list_name not in server_id_lists:
+                    deleted_lists.append(list_name)
 
-        # remove any list that has been deleted
-        for list_name in deleted_lists:
-            local_id_lists.pop(list_name, None)
+            # remove any list that has been deleted
+            for list_name in deleted_lists:
+                local_id_lists.pop(list_name, None)
+        except Exception as e:
+            self._errorBoundary.log_exception(e)
